/*
 *  File: IHierarchyIdService.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.util.ui.timebars.addon;

import de.jaret.util.ui.timebars.model.HierarchicalTimeBarModel;
import de.jaret.util.ui.timebars.model.TimeBarNode;

/**
 * For saving the viewstate nodes must be identified by an id and have to be found by the id. This interface describes a
 * service to do this for hierarchical models of the timebarviewer.
 * 
 * @author kliem
 * @version $Id: IHierarchyIdService.java 340 2007-04-02 21:52:03Z olk $
 */
public interface IHierarchyIdService {
    /**
     * Retrieve an id for a node.
     * 
     * @param model hierarchical model
     * @param node node
     * @return id for the node
     */
    String getId(HierarchicalTimeBarModel model, TimeBarNode node);

    /**
     * Get the node for a given id from the model.
     * 
     * @param model hierarchical model
     * @param id id
     * @return the node for the id or <code>null</code> if no node could be found
     */
    TimeBarNode getNode(HierarchicalTimeBarModel model, String id);
}
