/*
 *  File: TimeBarTableRowSorter.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.util.ui.timebars.addon.tablesynchronizer;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;

import de.jaret.util.ui.table.JaretTable;
import de.jaret.util.ui.timebars.AbstractTimeBarRowSorter;
import de.jaret.util.ui.timebars.model.TimeBarRow;

/**
 * TimeBarRowSorter, that bases its sorting on the row order of a TableTimeBarModel. This is used for synchronizing the
 * sorting in the direction from the table to the timebar viewer.
 * 
 * @author kliem
 * @version $Id: TimeBarTableRowSorter.java 470 2007-05-22 22:17:59Z olk $
 */
public class TimeBarTableRowSorter extends AbstractTimeBarRowSorter implements PropertyChangeListener, DisposeListener {
    /** the table widget. */
    private JaretTable _table;
    /** model wrapping the timeba model. */
    private TableTimeBarModel _ttbModel;

    /**
     * Construct the sorter.
     * 
     * @param ttbModel synchronizing model
     * @param table the table widget
     */
    public TimeBarTableRowSorter(TableTimeBarModel ttbModel, JaretTable table) {
        _table = table;
        _ttbModel = ttbModel;
        _table.addPropertyChangeListener(this);
    }

    /**
     * {@inheritDoc} Determine order by comparing the index in the table model.
     */
    public int compare(TimeBarRow o1, TimeBarRow o2) {
        TableTimeBarRow r1 = _ttbModel.getTableTimeBarRow(o1);
        TableTimeBarRow r2 = _ttbModel.getTableTimeBarRow(o2);

        int idx1 = _table.getInternalRowIndex(r1);
        int idx2 = _table.getInternalRowIndex(r2);

        return idx1 - idx2;
    }

    /**
     * {@inheritDoc} On disposal of the table, forget the reference.
     */
    public void widgetDisposed(DisposeEvent e) {
        _table = null;
    }

    /**
     * {@inheritDoc} When the sorting changes, inform the timebar viewer.
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals(JaretTable.PROPERTYNAME_SORTING)) {
            firePropertyChange("SORTING", null, "x");
        }
    }

}
