/*
 *  File: SwtHierachy.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.examples.timebars.addon.hierarchy.swt;

import java.text.DateFormat;
import java.util.Locale;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;

import de.jaret.examples.timebars.addon.hierarchy.model.DemoTimeBarNode;
import de.jaret.examples.timebars.addon.hierarchy.model.ModelCreator;
import de.jaret.util.date.JaretDate;
import de.jaret.util.date.holidayenumerator.HolidayEnumeratorFactory;
import de.jaret.util.ui.ResourceImageDescriptor;
import de.jaret.util.ui.table.JaretTable;
import de.jaret.util.ui.table.model.DefaultHierarchicalTableModel;
import de.jaret.util.ui.table.model.HierarchyColumn;
import de.jaret.util.ui.table.model.IColumn;
import de.jaret.util.ui.table.model.ITableViewState;
import de.jaret.util.ui.table.model.PropCol;
import de.jaret.util.ui.table.model.StdHierarchicalTableModel;
import de.jaret.util.ui.table.renderer.DateCellRenderer;
import de.jaret.util.ui.table.renderer.TableHierarchyRenderer;
import de.jaret.util.ui.timebars.TimeBarMarkerImpl;
import de.jaret.util.ui.timebars.addon.DefaultHierarchyIdService;
import de.jaret.util.ui.timebars.addon.IHierarchyIdService;
import de.jaret.util.ui.timebars.addon.tablesynchronizer.TableTimeBarNode;
import de.jaret.util.ui.timebars.addon.tablesynchronizer.TableTimeBarSynchronizer;
import de.jaret.util.ui.timebars.mod.DefaultIntervalModificator;
import de.jaret.util.ui.timebars.model.AddingTimeBarRowModel;
import de.jaret.util.ui.timebars.model.DefaultTimeBarNode;
import de.jaret.util.ui.timebars.model.HierarchicalTimeBarModel;
import de.jaret.util.ui.timebars.model.PPSInterval;
import de.jaret.util.ui.timebars.model.TimeBarRow;
import de.jaret.util.ui.timebars.swt.RowContextMenuHandler;
import de.jaret.util.ui.timebars.swt.TimeBarViewer;
import de.jaret.util.ui.timebars.swt.renderer.BoxTimeScaleRenderer;
import de.jaret.util.ui.timebars.swt.renderer.DefaultGridRenderer;
import de.jaret.util.ui.timebars.swt.renderer.DefaultHierarchyRenderer;
import de.jaret.util.ui.timebars.swt.renderer.DefaultTitleRenderer;
import de.jaret.util.ui.timebars.swt.util.actions.JaretTimeBarsActionFactory;

/**
 * Simple hierarchical view, SWT version. Scaling, manipulating the intervals, tree structure. The sum interval is
 * rendered by a specialized sum renderer. The arrows beetwenn the intervals are a demonstration of a GlobalAssistant
 * renderer. Context menus on scale and hierarchy. Hierarchy using label provider including icons.
 * <p>
 * The example also demonstrates the usage of a variable xaxis definition. To see that version set _variableXAxis to
 * true.
 * </p>
 * 
 * @author Peter Kliem
 * @version $Id: SwtHierachyWithTable.java 397 2007-05-01 12:39:10Z olk $
 */
public class SwtHierachyWithTable extends ApplicationWindow {

    static TimeBarViewer _tbv;

    public SwtHierachyWithTable() {
        super(null);
    }

    protected Control createContents(Composite parent) {
        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 1;
        parent.setLayout(gridLayout);

        // sash to place the two viewers on
        SashForm sashForm = new SashForm(parent, SWT.HORIZONTAL);
        GridData gd = new GridData(GridData.FILL_BOTH);
        sashForm.setLayoutData(gd);

        // create the model for the timebarviewer
        HierarchicalTimeBarModel model = ModelCreator.createModel(2, 5);

        // create the table model
        // first we need an id provider for mapping
        IHierarchyIdService idService = new DefaultHierarchyIdService();
        // creating the mapping nodes
        // Note: for synchronizing it is essential to use these nodes for mapping!
        TableTimeBarNode root = new TableTimeBarNode(idService, model, model.getRootNode());
        // and construct a hierarchical model around the mapped timebar nodes
        DefaultHierarchicalTableModel tableModel = new DefaultHierarchicalTableModel(root);

        // instantiate the table
        JaretTable table = new JaretTable(sashForm, SWT.H_SCROLL /* | SWT.V_SCROLL */); // no need for a vertical scroll
        // and set the model we just created
        table.setTableModel(tableModel);

        // now setup the table

        IColumn hcol = new HierarchyColumn();
        // create and setup hierarchy renderer
        TableHierarchyRenderer hierarchyRenderer = new TableHierarchyRenderer();
        hierarchyRenderer.setLabelProvider(new LabelProvider());
        hierarchyRenderer.setDrawIcons(true);
        hierarchyRenderer.setDrawLabels(true);
        table.registerCellRenderer(hcol, hierarchyRenderer);
        table.addColumn(hcol);
        table.getTableViewState().setColumnWidth(hcol, 200);
        // hierarchy column is fixed allowing easy row selections and leaving it in place.
        table.setFixedColumns(1);
        
        // add a column mapping the "text" property of the time bar node
        table.addColumn(new PropCol("text", "Text", "TimeBarNode.Text"));
        // add a column mapping the minimal date property of the time bar node (read only)
        PropCol column = new PropCol("min", "Min", "TimeBarNode.MinDate"); 
        column.setEditable(false);
        table.addColumn(column);
        // add a column mapping the maximal date property of the time bar node (read only)
        column = new PropCol("max", "Max", "TimeBarNode.MaxDate"); 
        column.setEditable(false);
        table.addColumn(column);

        
        // for the dates it is nicer to have the time in the display
        DateCellRenderer dcr = new DateCellRenderer();
        dcr.setDateformat(DateFormat.getDateTimeInstance());
        table.registerCellRenderer(JaretDate.class, dcr);
        
        
        table.getTableViewState().setRowHeightMode(ITableViewState.RowHeightMode.FIXED);


        
        // now create and set up the timebar viwer
        
        _tbv = new TimeBarViewer(sashForm, SWT.H_SCROLL | SWT.V_SCROLL);

        _tbv.setTimeScalePosition(TimeBarViewer.TIMESCALE_POSITION_TOP);

        _tbv.setModel(model);
        _tbv.setPixelPerSecond(0.00144);

        // configure the title renderer with a background image and set the title
        DefaultTitleRenderer titleRenderer = new DefaultTitleRenderer();
        titleRenderer.setBackgroundRscName("/de/jaret/examples/timebars/hierarchy/swt/titlebg.png");
        _tbv.setTitleRenderer(titleRenderer);
        _tbv.setTitle("SwtHierarchy");


        // create and configure hierarchy renderer
        DefaultHierarchyRenderer hrenderer = new DefaultHierarchyRenderer();
        hrenderer.setLabelProvider(new LabelProvider());
        hrenderer.setDrawLabels(true);
        hrenderer.setDrawIcons(true);
        hrenderer.setLevelWidth(25);
        hrenderer.setFixedLevelWidth(true);
        hrenderer.setRscNames("/de/jaret/examples/timebars/hierarchy/swt/collapsed.png",
                "/de/jaret/examples/timebars/hierarchy/swt/expanded.png",
                "/de/jaret/examples/timebars/hierarchy/swt/leaf.png");
        hrenderer.setSize(16);
        _tbv.setHierarchyRenderer(hrenderer);
        // do not show he hierarchy when starting
        // the hierarchy of the table will do
        _tbv.setHierarchyWidth(0);

        _tbv.setYAxisWidth(0);

        // allow interval modifications using the default modificator
        _tbv.addIntervalModificator(new DefaultIntervalModificator());

        // register sum renderer for merged intervals
        _tbv.registerTimeBarRenderer(AddingTimeBarRowModel.MergedInterval.class, new SumRenderer());

        // create and configure a grid renderer
        DefaultGridRenderer gridRenderer = new DefaultGridRenderer();
        gridRenderer.setHolidayEnumerator(HolidayEnumeratorFactory.getHolidayEnumeratorInstance(Locale.getDefault(),
                "NRW"));
        _tbv.setGridRenderer(gridRenderer);

        // time scale renderer
        _tbv.setTimeScaleRenderer(new BoxTimeScaleRenderer());

        final SimplePredecessorRenderer spr = new SimplePredecessorRenderer();
        _tbv.setGlobalAssistantRenderer(spr);

        // sample time bar marker
        TimeBarMarkerImpl tm = new TimeBarMarkerImpl(true, new JaretDate().advanceDays(1));
        tm.setDescription("TimeBarMarker");
        _tbv.addMarker(tm);

        // control panel
        SwtControlPanel ctrl = new SwtControlPanel(parent, SWT.NULL, _tbv, null);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        ctrl.setLayoutData(gd);

        // hierarchy control panel
        SwtHierarchyControlPanel ctrl2 = new SwtHierarchyControlPanel(parent, SWT.NULL, _tbv);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        ctrl2.setLayoutData(gd);

        // check double clicks: try to determine a link beeing clicked
        _tbv.addMouseListener(new MouseAdapter() {
            public void mouseDoubleClick(MouseEvent e) {
                DemoTimeBarNode hit = spr.hitPredecessor(e.x, e.y);
                System.out.println("HIT link to predecessor:" + (hit != null ? hit.getText() : "nothing"));
            }

        });

        // context menues
        // timescale context menu
        MenuManager mm = new MenuManager();
        mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_SCALETOWEEK));
        mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_SCALETOMONTH));
        mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_SCALETOYEAR));
        mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_CENTERSCALE));
        _tbv.setScaleContextMenu(mm.createContextMenu(_tbv));

        // row context menu
        RowContextMenuHandler hierarchyCtxHandler = new HCtxHandler();
        _tbv.setHierarchyCtxHandler(hierarchyCtxHandler);

        // instantiate the synchronizer
        TableTimeBarSynchronizer synchronizer = new TableTimeBarSynchronizer(table, _tbv);

        return _tbv;
    }

    public class HCtxHandler implements RowContextMenuHandler {
        Menu ctx;

        public HCtxHandler() {
            MenuManager mm = new MenuManager();
            mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_EXPANDNODE));
            mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_COLLAPSENODE));
            mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_EXPANDALL));
            mm.add(_tbv.getActionFactory().createStdAction(JaretTimeBarsActionFactory.ACTION_COLLAPSEALL));
            ctx = mm.createContextMenu(_tbv);
        }

        public Menu getContextMenu(TimeBarViewer tbv, TimeBarRow row) {
            return ctx;
        }
    }

    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        shell.setText(getClass().getName());
        shell.setSize(1000, 800);
    }

    public static void main(String[] args) {
        SwtHierachyWithTable test = new SwtHierachyWithTable();
        test.setBlockOnOpen(true);
        test.open();
    }

    /**
     * Label provider used in the table and the timebar viewer.
     * 
     * @author kliem
     * @version $Id: SwtHierachyWithTable.java 397 2007-05-01 12:39:10Z olk $
     */
    public class LabelProvider implements ILabelProvider {
        private ImageRegistry _imageRegistry;

        /**
         * {@inheritDoc}
         * Retrieve the image for a node. Take into account the timebarnodes AND the table timebar nodes.
         */
        public Image getImage(Object element) {
            if (element instanceof TableTimeBarNode) {
                element = ((TableTimeBarNode)element).getTimeBarNode();
            }
            
            if (element instanceof DemoTimeBarNode) {
                DemoTimeBarNode node = (DemoTimeBarNode) element;
                return node.isType() ? getImageRegistry().get("true") : getImageRegistry().get("false");
            } else {
                return getImageRegistry().get("adding");
            }

        }

        
        
        /**
         * {@inheritDoc}
         * Retrieve the text for a node. Take into account the timebarnodes AND the table timebar nodes.
         */
        public String getText(Object element) {
            if (element instanceof TableTimeBarNode) {
                element = ((TableTimeBarNode)element).getTimeBarNode();
            }
            if (element instanceof DemoTimeBarNode) {
                DemoTimeBarNode node = (DemoTimeBarNode) element;
                return node.getText();
            } else {
                return "adding node";
            }
        }

        public void addListener(ILabelProviderListener listener) {
        }

        public void dispose() {
            if (_imageRegistry != null) {
                _imageRegistry.dispose();
            }
        }

        public boolean isLabelProperty(Object element, String property) {
            return false;
        }

        public void removeListener(ILabelProviderListener listener) {
        }

        public ImageRegistry getImageRegistry() {
            if (_imageRegistry == null) {
                _imageRegistry = new ImageRegistry();
                ImageDescriptor imgDesc = new ResourceImageDescriptor("/de/jaret/examples/timebars/hierarchy/true.gif");
                _imageRegistry.put("true", imgDesc);
                imgDesc = new ResourceImageDescriptor("/de/jaret/examples/timebars/hierarchy/false.gif");
                _imageRegistry.put("false", imgDesc);
                imgDesc = new ResourceImageDescriptor("/de/jaret/examples/timebars/hierarchy/adding.gif");
                _imageRegistry.put("adding", imgDesc);
            }
            return _imageRegistry;
        }
    }

}