package de.jaret.examples.timebars.calendar.swt.renderer;

import java.text.DateFormatSymbols;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.printing.Printer;

import de.jaret.examples.timebars.calendar.model.Appointment;
import de.jaret.examples.timebars.calendar.model.AppointmentPlaceholder;
import de.jaret.examples.timebars.calendar.model.Day;
import de.jaret.examples.timebars.calendar.model.MonthTableModel;
import de.jaret.util.date.JaretDate;
import de.jaret.util.swt.SwtGraphicsHelper;
import de.jaret.util.ui.ResourceImageDescriptor;
import de.jaret.util.ui.table.JaretTable;
import de.jaret.util.ui.table.model.IColumn;
import de.jaret.util.ui.table.model.IRow;
import de.jaret.util.ui.table.renderer.CellRendererBase;
import de.jaret.util.ui.table.renderer.ICellRenderer;
import de.jaret.util.ui.table.renderer.ICellStyle;

public class DayCellRenderer extends CellRendererBase implements ICellRenderer {

    private static final int FIRSTLINEHEIGHT = 14;
    private static final int BARHEIGHT = 18;
    private static final int BARINSETS = 2;

    /** date format symbols instance. */
    protected static DateFormatSymbols _dateFormatSymbols;

    public DayCellRenderer(Printer printer) {
        super(printer);
        // TODO locale
        _dateFormatSymbols = new DateFormatSymbols(); // Java 6: .getInstance();
    }

    public DayCellRenderer() {
        this(null);
    }

    /**
     * {@inheritDoc}
     */
    public ICellRenderer createPrintRenderer(Printer printer) {
        return new DayCellRenderer(printer);
    }

    /**
     * {@inheritDoc}
     */
    public void dispose() {
        if (_clockImage != null) {
            _clockImage.dispose();
        }
        if (_imageRegistry != null) {
            _imageRegistry.dispose();
        }
    }

    /**
     * {@inheritDoc}
     */
    public void draw(GC gc, JaretTable jaretTable, ICellStyle cellStyle, Rectangle drawingArea, IRow row,
            IColumn column, boolean drawFocus, boolean selected, boolean printing) {

        Color fg = gc.getForeground();
        Color bg = gc.getBackground();
        Font font = gc.getFont();

        // get the value
        Day day = (Day) column.getValue(row);
        MonthTableModel monthTableModel = (MonthTableModel) jaretTable.getTableModel();

        // draw the default background
        drawBackground(gc, drawingArea, cellStyle, selected, printing);
        // if the day is not in the curent month draw a grey background
        Color background;
        if (day.getDayDate().getMonth() != monthTableModel.getMonth() + 1) { // +1 since month of the model is 0
            // based
            background = gc.getDevice().getSystemColor(SWT.COLOR_GRAY);
            gc.setBackground(background);
            gc.fillRectangle(drawingArea);
        }

        // cell borders and insets
        Rectangle drect = drawBorder(gc, cellStyle, drawingArea, printing);
        Rectangle rect = applyInsets(drect);

        // draw the line containing the date (right aligned)
        String label = getDayFirstLine(day.getDayDate());
        SwtGraphicsHelper.drawStringRightAlignedVTop(gc, label, rect.x + drect.width, rect.y);

        // if we are drawing the first of a month print the month name left aligned
        if (day.getDayDate().getDay() == 1) {
            String monthString = getMonthString(day.getDayDate().getMonth());
            gc.drawString(monthString, rect.x, rect.y);
        }

        // draw the spanning appointments
        Color bgx = gc.getBackground();
        int endy = rect.y + scaleY(FIRSTLINEHEIGHT);
        for (AppointmentPlaceholder ph : day.getPlaceholders()) {
            int y = rect.y + scaleY(FIRSTLINEHEIGHT) + ph.getPosition() * (scaleY(BARHEIGHT) + scaleY(1));

            if (y > endy) {
                endy = y;
            }

            gc.setBackground(gc.getDevice().getSystemColor(SWT.COLOR_YELLOW));

            gc.drawRectangle(drawingArea.x, y, drawingArea.width, scaleY(BARHEIGHT) - 1);
            if (ph.isFirst()) {
                gc.fillRectangle(drawingArea.x + 1, y + 1, drawingArea.width, scaleY(BARHEIGHT) - 2);
                // int sx = drawingArea.x + scaleX(INSETS);
                // if (!ph.getAppointment().isWholeDayAppointment()) {
                // gc.drawImage(getClockImage(gc.getDevice()), sx, y);
                // sx += getClockImage(gc.getDevice()).getBounds().width + scaleY(INSETS);
                // }
                if (ph.isLast()) {
                    gc.drawLine(drawingArea.x + drawingArea.width - 1, y + 1, drawingArea.x + drawingArea.width - 1, y
                            + scaleY(BARHEIGHT) - 2);
                }
                // SwtGraphicsHelper.drawStringVCentered(gc, ph.getAppointment().getText(), sx, y, y
                // + scaleY(APPOINTMENT_HEIGHT));
            } else if (ph.isLast()) {
                gc.fillRectangle(drawingArea.x, y + 1, drawingArea.width - 1, scaleY(BARHEIGHT) - 2);
            } else {
                gc.fillRectangle(drawingArea.x, y + 1, drawingArea.width, scaleY(BARHEIGHT) - 2);
            }

            // for every part of the appointment in the differnet rows paint the icon and and thet text
            // relying on the clipping on the gc
            IColumn startCol = monthTableModel.getColumnForDate(ph.getAppointment().getRealBegin());
            IRow startRow = monthTableModel.getRowForDate(ph.getAppointment().getRealBegin());
            if (startRow != null) {
                // only draw text if the appointment is in one row
                // TODO support multiple rows
                if (startRow.equals(row)) {
                    // TODO works for displayed columns only (may be sufficient)
                    Rectangle colRect = jaretTable.getColumnBounds(startCol);
                    if (colRect != null) {
                        int sx = colRect.x + scaleX(BARINSETS);
                        if (!ph.getAppointment().isWholeDayAppointment()) {
                            gc.drawImage(getClockImage(gc.getDevice()), sx, y);
                            sx += getClockImage(gc.getDevice()).getBounds().width + scaleY(BARINSETS);
                        }
                        SwtGraphicsHelper.drawStringVCentered(gc, ph.getAppointment().getText(), sx, y, y
                                + scaleY(BARHEIGHT));
                    }
                }
            }
            // draw selection using alpha blending
            // if (delegate.getSelectionModel().isSelected(ph.getAppointment())) {
            // Color bgx = gc.getBackground();
            // int alpha = gc.getAlpha();
            // gc.setBackground(gc.getDevice().getSystemColor(SWT.COLOR_BLUE));
            // gc.setAlpha(SELECTION_ALPHA);
            // gc.fillRectangle(drawingArea.x, y + 1, drawingArea.width, scaleY(APPOINTMENT_HEIGHT) - 2);
            //
            // gc.setAlpha(alpha);
            // gc.setBackground(bgx);
            // }

        }
        gc.setBackground(bgx);

        // draw the list of appointments
        endy += scaleY(BARHEIGHT) + scaleY(1);
        String appointmentListString = getAppointmentsString(day);
        gc.drawText(appointmentListString, rect.x, endy);

        if (drawFocus) {
            drawFocus(gc, drect);
        }
        drawSelection(gc, drawingArea, cellStyle, selected, printing);

        gc.setForeground(fg);
        gc.setBackground(bg);
        gc.setFont(font);

    }

    private String getAppointmentsString(Day day) {
        StringBuilder buf = new StringBuilder();

        for (Appointment appointment : day.getAppointments()) {
            if (!appointment.isSpansMultipleDays() && !appointment.isWholeDayAppointment()) {
                buf.append(appointment.getBegin().toDisplayStringTime(false));
                buf.append(" ");
                buf.append(appointment.getEnd().toDisplayStringTime(false));
                buf.append(" ");
                buf.append(appointment.getText());
                buf.append("\n");
            }
        }

        return buf.toString();
    }

    /**
     * Retrieve the textual label for a month.
     * 
     * @param month the month (1 based)
     * @return localized month name
     */
    private String getMonthString(int month) {
        return _dateFormatSymbols.getMonths()[month - 1];
    }

    /**
     * {@inheritDoc}
     */
    public String getTooltip(JaretTable jaretTable, Rectangle drawingArea, IRow row, IColumn column, int x, int y) {
        Day day = (Day) column.getValue(row);

        return day.getDayDate().toDisplayStringDate() + "\n" + getAppointmentsString(day);
    }

    /**
     * Generate the String to be shown in the first line of the day cell.
     * 
     * @param date date to be rendered
     * @return formatted String
     */
    protected String getDayFirstLine(JaretDate date) {
        return date.getDay() + ".";
    }

    private Image _clockImage;
    /** image registry for holding the icon. */
    protected ImageRegistry _imageRegistry;

    private Image getClockImage(Device device) {
        if (_clockImage != null) {
            return _clockImage;
        }
        _clockImage = getImageRegistry().getDescriptor("clock").createImage();
        return _clockImage;
    }

    /**
     * Retrieve the image registry (lazy creation).
     * 
     * @return the initialized registry
     */
    protected ImageRegistry getImageRegistry() {
        if (_imageRegistry == null) {
            _imageRegistry = new ImageRegistry();
            ImageDescriptor imgDesc = new ResourceImageDescriptor(
                    "/de/jaret/examples/timebars/calendar/swt/renderer/clock.gif");
            _imageRegistry.put("clock", imgDesc);
        }
        return _imageRegistry;
    }

}
